" SwitchColor plugin. h SwitchColor
" Copyright (c) 2022 Tommy Bollman/McUsr Vim Licence.
" version 0.0.4 Removed problem with regards to mapping of functions.
if v:version < 700 || exists('g:loaded_switchcolor') || &cp
 	finish
endif
let g:loaded_switchcolor = 1
" ================================================================
" Keybindings. Smartest to leave alone, and remap in vimrc.
" ================================================================

execute "set <M-c>=\ec"
nmap <unique><silent><M-c> :colorscheme <C-D>

execute "set <M-c>=\eg"
nmap <unique><silent><M-g> :colorscheme <cr>

if !hasmapto('<Plug>SwitchColorForward;')
	execute "set <M-6>=\e6"
	map <unique> <M-6> <Plug>SwitchColorForward;
endif

if !hasmapto('<Plug>SwitchColorBackward;')
	execute "set <M-5>=\e5"
	map <unique> <M-5> <Plug>SwitchColorBackward;
endif

noremap <unique> <script> <Plug>SwitchColorForward; <SID>SCSwitchColorForward

noremap <unique> <script> <Plug>SwitchColorBackward; <SID>SCSwitchColorBackward 

noremap <SID>SCSwitchColorForward  :call SCSwitchColorForward()<CR>
noremap <SID>SCSwitchColorBackward :call SCSwitchColorBackward()<CR>

let s:setupForSwitching = 0
" Constant values.
let s:validschemes = [ 'whole_runtime', 'local_only', 'user_specified' ]
" ================================================================
" User functions you may change According to your liking
" ================================================================
function! SCLight()
	" My light backgrounds.
	let g:SC_schemesforuse  = 'user_specified'
	let g:SC_colorscheme_place = '$HOME/vimfiles/color/light'
	call  SCParseUserValues()
endfunction

function! SCDark()
	" My light backgrounds.
	let g:SC_schemesforuse  = 'user_specified'
	let g:SC_colorscheme_place = '$HOME/vimfiles/color/dark'
	call  SCParseUserValues()
endfunction

function! SCNormal()
	" My light backgrounds.
	let g:SC_schemesforuse  = 'local_only'
	call  SCParseUserValues()
endfunction
" ================================================================
" Tier with functions for parsing global into variables.
" ================================================================
function! SCDebugInfo()
" To be run AFTER SCGetUserValues() to check values.
	if (s:setupForSwitching == 0 )
		call SCParseUserValues()
		call SCSetInitColScheme() 
	endif 
	execute "echo \"s:schemesforuse:\"     " .. "s:schemesforuse"
	execute "echo \"s:fileCurColor:\"       " .. "s:fileCurColor"    
	execute "echo \"s:fallback_scheme:\" " .. "s:fallback_scheme" 
	execute "echo \"s:favcolschemes:\"     " .. "s:favcolschemes"   
	execute "echo \"g:CURCOLOR:\"             " .. "g:CURCOLOR"
endfunction

function! SCGetUserValues()
	" Called by SCParseUserValues()
	" We come here allways, after we have initialized the  
	" internal fall back er "whole_runtime"
	let s:schemesforuse          = get(g:, 'SC_schemesforuse', 'whole_runtime')
	if  (index( s:validschemes,s:schemesforuse) == -1 )
		echoerr "SwitchColor: wrong value in g:SC_schemesforuse,uses 'whole_runtime'"
		let s:schemesforuse        = 'whole_runtime' " FALLBACK
	endif

  if s:schemesforuse == 'user_specified' 
		let s:favcolschemes   		 = get(g:, 'SC_colorscheme_place', "")
		if s:favcolschemes == ""
			echoerr "SwitchColor: no value in g:SC_colorscheme_place,uses
						\ g:SC_schemesforuse = 'whole_runtime' as FALLBACK"
			let s:schemesforuse        = 'whole_runtime' " FALLBACK
		endif 
		" we need to check if this works, and if not, we fall back to runtime.
	endif
	let s:rejects 							 = get(g:, 'SC_excluded', [] )
	let s:fallback_scheme        = get(g:, 'SC_colosch_fback', "desert")

	if !exists('g:SC_dotfile_name')
		" shamelessly stolen from Y. Lakshmanan's MRU plugin.
			if has('unix') || has('macunix')
					let s:fileCurColor = $HOME . '/.curcolors'
			else
					let s:fileCurColor = $VIM . '/_curcolors'
					if has('win32')
							" MS-Windows
							if $USERPROFILE != ''
									let s:fileCurColor = $USERPROFILE . '\_curcolors'
							endif
					endif
			endif
	else
			let s:fileCurColor=expand(g:SC_dotfile_name)
	endif
endfunction

" initial values:
let s:ixCurScheme = 0
let s:swback = 0

function! SCParseUserValues()
	call SCGetUserValues()
	if s:schemesforuse  == 'whole_runtime' 
		let paths = split(globpath(&runtimepath, 'colors/*.vim'), "\n")
	elseif s:schemesforuse  == 'local_only' 
		let paths = split(globpath('~/.vim/', 'colors/*.vim'), "\n")
	else " user_specified
		let paths = split(globpath(s:favcolschemes, '/*.vim'), "\n")
	endif

	let s:swcolors = map(paths, 'fnamemodify(v:val, ":t:r")')
	call sort(s:swcolors)
	let s:swback = 0    " background variants light/dark was not yet switched
	let s:ixCurScheme = index(s:swcolors,g:CURCOLOR)

	if s:ixCurScheme == -1 
	" Scenario is that we have changed collection of colors themes, eg. from
	" dark to light, or to c-syntax from html.
		let s:ixCurScheme = 1
		let g:CURCOLOR = s:swcolors[s:ixCurScheme]
	endif 
endfunction
" ================================================================
" Functions for reading/writing current colorscheme to/from disk.
" ================================================================
function! SCAssertCURCOLOR()
" Asserts we have a value, before writing, and reading
	let l:probe=""
	try
		let l:probe = g:CURCOLOR
	catch /^Vim\%((\a\+)\)\=:E/	 " catch all Vim errors
	endtry 
	if l:probe == ""
		let g:CURCOLOR = s:fallback_scheme 
	endif
endfunction 

function! SCSourceCURCOLOR()
" To be installed on the VimEnter
  call SCParseUserValues()
	let l:colorFile  = expand(s:fileCurColor)
	try
		let l:firstL = readfile(l:colorFile ,'',1 )
		let l:firstLs	=	string(l:firstL[0])
		let g:CURCOLOR = substitute(l:firstLs, "'", '', 'g')
	catch /^Vim\%((\a\+)\)\=:E/	 " catch all Vim errors
	endtry
	call SCSetInitColScheme()
endfunction

function! SCFlushCURCOLOR()
" to be installed on VimLeave call's SCParseUserValues() 
	call SCAssertCURCOLOR()
	let	l:colorFile  = expand(s:fileCurColor)
	call writefile([g:CURCOLOR], l:colorFile, "S")
endfunction
" ================================================================
" Functions that lets us choose/loop through colors.
" ================================================================
function! SCSetInitColScheme()
 " SCParseUserValues allways called up front. 
	if (s:setupForSwitching == 0 )
		call SCAssertCURCOLOR()
		execute "colorscheme " .. g:CURCOLOR
	endif
	let s:setupForSwitching = 1
endfunction

function! SCCurrentScheme()
	let l:scrapval = ""
	redir => l:scrapval
		silent execute "colorscheme"
	redir END
	return substitute(l:scrapval, '^\n*', '', '')
endfunction

function! SCRefreshColor()
	let g:CURCOLOR = SCCurrentScheme()
	let cmdStr = "colorscheme " .. g:CURCOLOR
	silent execute cmdStr 
endfunction

" ================================================================
" Actual switching between schemes
" ================================================================

function! SCSwitchColorForward()
	call SCSwitchColor(1)
endfunction

function! SCSwitchColorBackward()
	call SCSwitchColor(-1)
endfunction

function! SCSwitchColor(swinc)
	let l:wasLight = 0
	let g:CURCOLOR = SCCurrentScheme()
	" We start from the right place.
	let s:ixCurScheme = index(s:swcolors,g:CURCOLOR)
	" if have switched background: dark/light
	let l:oldix = s:ixCurScheme
	while 1
		if (l:oldix == s:ixCurScheme) && (&background == "dark") 
			execute "set background=light"
			if exists('g:colors_name')
				" vim global set when successful change.
				break
			else
				let s:ixCurScheme = s:ixCurScheme + a:swinc
				let l:oldidx = -2 
			endif
		elseif (l:oldix == s:ixCurScheme) && ( &background == "light") 
			let l:wasLight = 1
			execute "set background=dark"
			let s:ixCurScheme = s:ixCurScheme + a:swinc
			let l:oldix = -2
		else
			" we are about to change the colorscheme
			" the scheme is black. 
			let s:ixCurScheme = s:ixCurScheme % len(s:swcolors)
			if (s:ixCurScheme < 0 ) && (a:swinc < 0 )
				let s:ixCurScheme = len(s:swcolors) - 1
			endif 
			call sort(s:rejects)
			" if not in rejects list
			if (index(s:rejects, s:swcolors[s:ixCurScheme]) == -1)
				execute "colorscheme " .. s:swcolors[s:ixCurScheme]
				if (l:wasLight == 1)  && (a:swinc < 0)
					execute "set background=light"
					if (!exists('g:colors_name'))
						execute "set background=dark"
						execute "colorscheme " .. s:swcolors[s:ixCurScheme]
						let l:wasLight = 0
					endif
				endif
				let g:CURCOLOR = s:swcolors[s:ixCurScheme]
				break
			else
				let s:ixCurScheme = s:ixCurScheme + a:swinc 
			endif
		endif
	endwhile
	" show current name on screen. :h :echo-redraw
	redraw
	execute "colorscheme"
endfunction
" ================================================================
" Event handler that lets previous colorscheme be set
" Saving of colorscheme upon exit
" Making files allways be selectable in Netrw.
" ================================================================

let initColScheme = 0
augroup setupColorScheme
  autocmd!
  autocmd! VimEnter * call SCSourceCURCOLOR()
	autocmd! WinEnter * if initColScheme == 0 | call SCParseUserValues() | call SCSetInitColScheme() | let initColScheme = 1 | endif  
 	autocmd! VimLeave * call SCFlushCURCOLOR()
	autocmd! FileType netrw set nocursorline | call SCRefreshColor()
augroup END
